﻿using System;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace CoreSharp.Keyboard.Layouts
{
    public class ThaiKeyboardLayout : KeyboardLayout
    {
		#region Singleton Pattern
			
		private static ThaiKeyboardLayout _Default;

		private ThaiKeyboardLayout()
		{
		}

		///<summary>
		/// Get the default instance of ThaiKeyboardLayout
		///</summary>
		public static ThaiKeyboardLayout Default
		{
			get
			{
				if ( _Default == null )
				{
					_Default = new ThaiKeyboardLayout();
				}
				return _Default;
			}
		}
		
		#endregion

        public override string LayoutIcon
        {
            get { return null; }
        }

        public override string LayoutName
        {
            get { return "th"; }
        }

        private char[] _Row1 = "ภถคตจขชล".ToCharArray();
        private char[] _Row2 = "ๆไำพะรนยบ".ToCharArray();
        private char[] _Row3 = "ฟหกดเาสวง".ToCharArray();
        private char[] _Row4 = "ผปแอทมใฝ".ToCharArray();

        public override char[] Row1
        {
            get { return _Row1; }
        }

        public override char[] Row2
        {
            get { return _Row2; }
        }

        public override char[] Row3
        {
            get { return _Row3; }
        }

        public override char[] Row4
        {
            get { return _Row4; }
        }

        public override KeyboardLayout ShiftKeyboardLayout
        {
            get
            {
                return ThaiShiftKeyboardLayout.Default;
            }
        }

        public override KeyboardLayout AltKeyboardLayout
        {
            get
            {
                return ThaiVowelConsonantKeyboardLayout.Default;
            }
        }

    }
}
